extern crate core;

mod helpers;
mod middlewares;
mod models;
mod routes;
mod schema;
mod services;
mod utils;

use crate::helpers::db;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    use actix_web::{web, App, HttpServer};

    dotenvy::dotenv().expect("No .env file found");
    std::env::set_var("RUST_LOG", "debug");

    // Register services.
    let _ = services::spotify::instance().await;

    db::init();

    HttpServer::new(move || {
        App::new()
            .service(web::scope("/playlists").service(routes::playlists::get_playlist))
            .service(routes::auth::routes())
            .service(routes::me::routes())
            .service(routes::users::routes())
            .service(routes::search::routes())
    })
    .bind(("127.0.0.1", 9000))?
    .run()
    .await
}
