use crate::helpers::jwt::get_token;
use crate::middlewares::error::ErrorResponse;
use crate::models::user::Users;
use actix_web::http::{header, StatusCode};
use actix_web::HttpRequest;

pub fn get_user(req: HttpRequest) -> Result<Users, ErrorResponse> {
    let authorization = req.headers().get(header::AUTHORIZATION);

    match authorization {
        Some(header) => {
            let token_data = get_token(header.to_str().unwrap());

            match token_data {
                Ok(token_data) => {
                    let user = Users::find(token_data.claims.user_id.as_str())?;

                    Ok(user)
                }
                Err(e) => {
                    return Err(match e.kind() {
                        jsonwebtoken::errors::ErrorKind::ExpiredSignature => {
                            ErrorResponse::new("Not Authorized", StatusCode::UNAUTHORIZED)
                        }
                        _ => ErrorResponse::new(
                            e.to_string().as_str(),
                            StatusCode::INTERNAL_SERVER_ERROR,
                        ),
                    })
                }
            }
        }
        None => Err(ErrorResponse::new(
            "Not Authorized",
            StatusCode::UNAUTHORIZED,
        )),
    }
}
