use crate::middlewares::error::ErrorResponse;
use crate::models::playlists::{PlaylistCreator, Playlists};
use crate::models::tracks::TracksWithArtists;
use actix_web::{get, web, HttpResponse, Scope};
use serde::{Deserialize, Serialize};

pub fn routes() -> Scope {
    web::scope("/playlists").service(get_playlist)
}

#[derive(Deserialize, Serialize)]
struct GetPlaylistResponse {
    pub id: String,
    pub name: String,
    pub creator: PlaylistCreator,
    pub tracks: Vec<TracksWithArtists>,
    pub tracks_count: usize,
    pub duration: usize,
}

#[get("/{playlist_id}")]
async fn get_playlist(path: web::Path<String>) -> Result<HttpResponse, ErrorResponse> {
    let playlist_id = path.into_inner();
    let playlist = Playlists::find(playlist_id.as_str())?;

    let creator = playlist.get_creator()?;

    let tracks = playlist.get_tracks()?;
    let tracks_count = tracks.len();
    let duration = tracks
        .iter()
        .map(|track| track.duration_ms)
        .reduce(|a, b| a + b)
        .unwrap() as usize;

    Ok(HttpResponse::Ok().json(GetPlaylistResponse {
        id: playlist.id.to_string(),
        name: playlist.name.to_string(),
        creator: PlaylistCreator {
            id: creator.id,
            name: creator.name,
        },
        tracks,
        tracks_count,
        duration,
    }))
}
