use crate::middlewares::error::ErrorResponse;
use crate::models::spotify;
use crate::services::spotify as Spotify;
use actix_web::{get, web, HttpResponse, Result, Scope};
use serde::{Deserialize, Serialize};

pub fn routes() -> Scope {
    web::scope("/search").service(search)
}

#[derive(Deserialize)]
struct SearchQuery {
    q: String,
}

#[derive(Serialize)]
struct SearchResponse {
    tracks: Vec<SearchTracks>,
}

#[derive(Serialize)]
struct SearchTracks {
    pub id: String,
    pub title: String,
    // pub artists: String,
    pub duration_ms: i32,
}

impl From<spotify::Track> for SearchTracks {
    fn from(value: spotify::Track) -> SearchTracks {
        SearchTracks {
            id: value.id,
            title: value.name,
            duration_ms: value.duration_ms,
        }
    }
}

#[get("")]
async fn search(query: web::Query<SearchQuery>) -> Result<HttpResponse, ErrorResponse> {
    let spotify = Spotify::instance().await;
    let search = spotify.search(&query.q).await.unwrap();

    Ok(HttpResponse::Ok().json(SearchResponse {
        tracks: search
            .tracks
            .items
            .into_iter()
            .map(SearchTracks::from)
            .collect::<Vec<SearchTracks>>(),
    }))
}
